<?php

function deadtome_kill_list($account) {
  $destination = drupal_get_destination();

  $header = array(
    'user' => array('data' => t('Victim')),
    'killed_on' => array('data' => t('Killed on'), 'field' => 'killed_on', 'sort' => 'desc'),
    'operations' => array('data' => t('Operations')),
  );

  $query = db_select('deadtome')->extend('TableSort');
  $query->fields('deadtome');
  $query->condition('uid', $account->uid);
  $query->orderByHeader($header);
  $records = $query->execute()->fetchAllAssoc('dead_uid');

  $accounts = user_load_multiple(array_keys($records));

  $rows = array();
  foreach ($records as $record) {
    $record->target = $accounts[$record->dead_uid];
    $row = array();
    $row[] = check_plain(format_username($record->target));
    $row[] = !empty($record->killed_on) ? format_date($record->killed_on) : t('Unknown');
    $row[] = l(t('Resurrect'), deadtome_get_url($record->target, 'resurrect', $account), array('query' => $destination));
    $rows[] = $row;
  }

  $page['records'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => $account->uid == $GLOBALS['user']->uid ? t('You have not killed anyone... yet.') : t("@name has not killed anyone... yet.", array('@name' => format_username($account))),// . ' ' . l(t('Kill a user.'), "user/{$account->uid}/deadtome/kill", array('query' => $destination)),
  );

  $page['kill'] = drupal_get_form('deadtome_kill_form');
  $page['kill']['target']['#title'] = t('Would you like to kill someone?');
  $page['kill']['actions']['cancel']['#access'] = FALSE;
  $page['kill']['#access'] = ($account->uid === $GLOBALS['user']->uid);

  return $page;
}

function deadtome_kill_form($form, $form_state) {
  $form['target'] = array(
    '#type' => 'value',
    '#value' => NULL,
  );

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Who do you want to kill?'),
    '#maxlength' => 60,
    '#autocomplete_path' => 'deadtome/autocomplete',
    '#default_value' => '',
    '#required' => TRUE,
    '#attributes' => array(
      'placeholder' => t("Your victim's name"),
    ),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Kill'),
  );
  $form['actions']['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => isset($_GET['destination']) ? $_GET['destination'] : "user/{$GLOBALS['user']->uid}/deadtome",
  );

  return $form;
}

function deadtome_kill_form_validate($form, &$form_state) {
  $target = user_load_by_name($form_state['values']['name']);

  if (empty($target)) {
    form_set_error('name', t('User %name not found.', array('%name' => $form_state['values']['name'])));
  }
  elseif (!deadtome_access($target, 'kill')) {
    if ($target->uid == $GLOBALS['user']->uid) {
      form_set_error('name', t('You cannot kill yourself. This is not the suicide module.'));
    }
    elseif (deadtome_user_is_dead($target->uid)) {
      form_set_error('name', t('You have already killed this user.'));
    }
    else {
      form_set_error('name', t('You cannot kill this user.'));
    }
  }
  else {
    // Set the target user's UID value back into the form.
    $form_state['values']['target'] = $target;
  }
}

function deadtome_kill_form_submit($form, &$form_state) {
  $account = $GLOBALS['user'];
  $target = $form_state['values']['target'];
  deadtome_kill_users($account, array($target->uid));
  drupal_set_message(t('You have killed %user, who is now dead to you.', array('%user' => format_username($target))));
  $form_state['redirect'] = "user/{$account->uid}/deadtome";
}

/**
 * Menu callback to kill a user.
 */
function deadtome_kill_user($account, $target, $token) {
  if (!drupal_valid_token($token, "user/{$account->uid}/deadtome/kill/{$target->uid}")) {
    return MENU_ACCESS_DENIED;
  }

  deadtome_kill_users($account, array($target->uid));
  drupal_set_message(t('You have killed %user, who is now dead to you.', array('%user' => format_username($target))));
  drupal_goto("user/{$account->uid}/deadtome");
}

/**
 * Menu callback to resurrect a user.
 */
function deadtome_resurrect_user($account, $target, $token) {
  if (!drupal_valid_token($token, "user/{$account->uid}/deadtome/resurrect/{$target->uid}")) {
    return MENU_ACCESS_DENIED;
  }

  deadtome_resurrect_users($account, array($target->uid));
  drupal_set_message(t('You have resurrected %user, who is now undead to you.', array('%user' => format_username($target))));
  drupal_goto("user/{$account->uid}/deadtome");
}

function deadtome_user_autocomplete($string = '') {
  $matches = array();

  if ($string) {
    $account = user_load($GLOBALS['user']->uid);

    $query = db_select('users', 'u');
    $query->fields('u', array('name'));
    $query->condition('u.name', db_like($string) . '%', 'LIKE');
    // Exclude the current user from the list of results.
    $query->condition('u.uid', $account->uid, '<>');
    // Exclude any already killed users from the results.
    if (!empty($account->deadtome)) {
      $query->condition('u.uid', array_keys($account->deadtome), 'NOT IN');
    }
    // Exclude any users that have a role which cannot be killed.
    if ($exclude_roles = user_roles(FALSE, 'is unkillable')) {
      $query->leftJoin('users_roles', 'ur', 'u.uid = ur.uid');
      $or = db_or();
      $or->condition('ur.rid', array_keys($exclude_roles), 'NOT IN');
      $or->condition('ur.rid', NULL, 'IS NULL');
      $query->condition($or);
    }
    $query->range(0, 10);

    $matches = $query->execute()->fetchAllKeyed(0, 0);
    $matches = array_map('check_plain', $matches);
  }

  drupal_json_output($matches);
}
